GitLab CI settings
==================

- [Root .gitlab-ci.yml](../.gitlab-ci.yml)
  - This loads actual settings defined in this directory using [include:local](https://docs.gitlab.com/ee/ci/yaml/#includelocal) feature
- You should open [GitLab CI YAML reference](https://docs.gitlab.com/ee/ci/yaml/) when editing this directory.
- monolish uses GitLab CI heavily for better developing process, but it is accessible only for RICOS members.

Stages
------

monolish CI utilizes [needs](https://docs.gitlab.com/ee/ci/yaml/#needs) feature to represent dependencies between jobs,
and the stage ordering does not represent execution order.
Independent jobs run in parallel as far as usable runner exists.
Following stages exist only for semantic grouping.

### develop

- [develop.yml](./develop.yml)

Support jobs to keep this repository healthy.

- clang-format
  - Keep C++ codes are well-formatted
- doxygen
  - Generate document HTML to review in each Merge Request using [Review Apps](https://docs.gitlab.com/ee/ci/review_apps/)
- markdown-link-check
  - Check links in README live
- keep-changelog
  - Not to forget updating [CHANGELOG.md](../CHANGELOG.md)

### build

- [build.yml](./build.yml)
- [build_generated.yml](./build_generated.yml)
  - Generated by [case_generator.py](./case_generator.py)

Responsible for compiling C++ code in `src/` into a shared library and for compiling `test/` codes.
This can be run in CPU runners since compiler does not requires GPU,
although running output executable requires GPU.
Generated library and test executables are passed to next stages as artifacts.

### test

- [test.yml](./test.yml)

Run executables in [test/](../test) directory built in the build step.

### example

- [example.yml](./example.yml)

Build and run executables in [examples/](../examples) directory using deb file built in the build step.
Different from test step, it compiles example codes.

### benchmark

- [benchmark.yml](./benchmark.yml)

### python

- [python.yml](./python.yml)

Test log handling tools developed in [python/](../python).

### package

- [package.yml](./package.yml)
- [package_generated.yml](./package_generated.yml)
  - Generated by [case_generator.py](./case_generator.py)

Generate all deb files for release.
Since the build step generates deb files for testing, all necessary deb files are not built.

### deploy

- [docker.yml](./docker.yml)
- [docker_generated.yml](./docker_generated.yml)
  - Generated by [case_generator.py](./case_generator.py)

Build and push docker images both to [registry.ritc.jp/ricos/monolish](https://gitlab.ritc.jp/ricos/monolish/container_registry) and to [GitHub Container Registry (ghcr.io)](https://github.com/orgs/ricosjp/packages?repo_name=monolish).
